//File Description: Keep track of hot key actions

//Pointer types used in other config files that are not used here
typedef void GtkWidget;

#include <gdk/gdkkeys.h> //Convert the key names
#include <glib/gstring.h>
#include <glib/glist.h>
#include <glib/gstrfuncs.h>
#include <malloc.h>
#include "Actions.h"
#include "OSInterface.h"
#include "Environment.h"
#include "Config.h"

//Default Settings. The Key.Key value is assumed to be a KeyName and is converted to a hardware keycode in place by GTK during GetDefaultSettings
static HotKeyAction DefaultSettings[5]={ //Need to be able to modify the Key.Key, so do not set as const
	{{'a',MODK_CONTROL|MODK_SHIFT}, FALSE, HKACTIONTYPE_SetStatus, {STATUS_AWAY,      SSMS_STATIC, "afk or unavailable"},0},
	{{'b',MODK_CONTROL|MODK_SHIFT}, FALSE, HKACTIONTYPE_SetStatus, {STATUS_AVAILABLE, SSMS_STATIC, ""                  },0},
	{{'d',MODK_CONTROL|MODK_SHIFT}, FALSE, HKACTIONTYPE_SetStatus, {STATUS_OFFLINE,   SSMS_STATIC, ""                  },0},
	{{'s',MODK_CONTROL|MODK_SHIFT}, FALSE, HKACTIONTYPE_SetStatus, {STATUS_AWAY,      SSMS_POPUP,  "afk"               },0},
	{{'t',MODK_CONTROL|MODK_SHIFT}, FALSE, HKACTIONTYPE_Other,     {OHK_FOCUS,        0,  NULL                         },0}
};

GList *Actions=NULL; //The list of actions

//Internal functions: Get/Set/Delete action related preferences
static const char* FormatAPrefName   (int ActionNum, const char* ActionMember, GString *TempString);                       //Formats the preference name
static int         GetIntFromAPref   (int ActionNum, const char* ActionMember, GString *TempString);                       //Retreive an integer
static const char* GetStringFromAPref(int ActionNum, const char* ActionMember, GString *TempString);                       //Retreive a string
static void        SetAPrefFromInt   (int ActionNum, const char* ActionMember, GString *TempString, int         NewValue); //Save an integer
static void        SetAPrefFromString(int ActionNum, const char* ActionMember, GString *TempString, const char* NewValue); //Save a string
static void        RemoveAPref       (int ActionNum, const char* ActionMember, GString *TempString);                       //Delete a preference

//Saves a Hotkey action
void SaveAction(HotKeyAction *Action, gboolean SaveToPreferences)
{
	//Make sure the key hasn't already been bound locally
	gboolean IsKeyAlreadyUsed=FALSE;
	HotKeyAction* TempHKA;
	for(GList *TraverseActions=Actions;TraverseActions;TraverseActions=g_list_next(TraverseActions)) //Check each current action
		if((TempHKA=(HotKeyAction*)TraverseActions->data)->Key.Key==Action->Key.Key && TempHKA->Key.Modifiers==Action->Key.Modifiers)
			IsKeyAlreadyUsed=TRUE;

	//Add the new action onto the actions list
	guint ActionNumber=g_list_length(Actions);
	Actions=g_list_append(Actions, Action);

	//Save to preferences if requested
	GString *TempString=g_string_new(NULL);
	if(SaveToPreferences)
	{
		PrefsSetInt("Num", ActionNumber+1);
		SetAPrefFromInt(ActionNumber, "Key", TempString, Action->Key.Modifiers<<16|Action->Key.Key);
		SetAPrefFromInt(ActionNumber, "Type", TempString, Action->Type);
		if(Action->Type==0) //HKACTIONTYPE_SetStatus
		{
			SetAPrefFromInt(ActionNumber, "State", TempString, Action->Info.State);
			SetAPrefFromInt(ActionNumber, "MessageState", TempString, Action->Info.MessageState);
			SetAPrefFromString(ActionNumber, "Message", TempString, Action->Info.Message);
		}
		else //HKACTIONTYPE_Other
			SetAPrefFromInt(ActionNumber, "Action", TempString, Action->Info.State);
	}
	g_string_free(TempString, TRUE);

	//Register the global hot key
	Action->Identifier=(IsKeyAlreadyUsed ? 0 : RegisterHotKeyAction(ActionNumber, Action->Key.Key, Action->Key.Modifiers)); //Only try registering the key if it has not already been used locally
	if(Action->Identifier) //If successful, nothing left to do
		return;

	//Show an error dialog
	GString *TheKey=FormatKeyName(Action->Key);
	GString *ActionText=GetActionText(Action);
	GString *FormatText=g_string_new(NULL);
	g_string_printf(FormatText, "The following action's hot key was unable to be set\n%s\n%s", TheKey->str, ActionText->str);
	ShowDialogBox("Status Hot Keys", FormatText->str);

	//Free error dialog strings
	g_string_free(TheKey, TRUE);
	g_string_free(ActionText, TRUE);
	g_string_free(FormatText, TRUE);
}

//Deletes a Hotkey action
void DeleteAction(int ActionNumber, gboolean RemoveFromPreferences)
{
	GString *TempString=g_string_new(NULL);

	//Move backwards through the actions list and temporarily unhook all items up to the current item (which is permanently unset)
	GList *TempList=NULL;
	for(int i=g_list_length(Actions)-1;i>=ActionNumber;i--)
	{
		//Find the last action
		GList *LastItem=g_list_last(Actions); //Get the link in the list for the last action
		HotKeyAction *Action=(HotKeyAction*)LastItem->data; //Remember that actions data

		//Unregister the global hot key
		if(Action->Identifier) //Only do so if original register was successful
			UnRegisterHotKeyAction(Action->Identifier);

		//Move the last action from the Actions list to a temporary list, and get its data
		Actions=g_list_delete_link(Actions, LastItem); //Delete the link from the Actions list
		if(i!=ActionNumber) //Store onto the temporary list if it is not the action being unset
			TempList=g_list_prepend(TempList, Action);

		//Remove from preferences if requested
		if(RemoveFromPreferences)
		{
			PrefsSetInt("Num", i);
			RemoveAPref(i, "Key", TempString);
			RemoveAPref(i, "Type", TempString);
			if(Action->Type==0) //HKACTIONTYPE_SetStatus
			{
				RemoveAPref(i, "State", TempString);
				RemoveAPref(i, "MessageState", TempString);
				RemoveAPref(i, "Message", TempString);
			}
			else //HKACTIONTYPE_Other
				RemoveAPref(i, "Action", TempString);
		}

		//If this is the action being unset, delete it and its data
		if(i==ActionNumber)
		{
			if(Action->Type==0)
				g_free(Action->Info.Message);
			free(Action);
		}
	}

	//Iterate back through the temporary list and readd all the temporarily removed actions
	for(GList *TraverseActions=TempList;TraverseActions;TraverseActions=g_list_next(TraverseActions))
		SaveAction((HotKeyAction*)TraverseActions->data, RemoveFromPreferences);
	g_list_free(TempList);

	g_string_free(TempString, TRUE);
}

//Load an action's information from preferences
HotKeyAction* LoadActionFromPref(int ActionNumber)
{
	GString *TempString=g_string_new(NULL);
	HotKeyAction *Action=(HotKeyAction*)malloc(sizeof(HotKeyAction));

	//Extract the key (3 bytes for Key, 1 byte for Modifiers)
	int TempInt=GetIntFromAPref(ActionNumber, "Key", TempString);
	Action->Key.Key=(guint16)TempInt;
	Action->Key.Modifiers=TempInt>>16;

	//Extract the preference information
	Action->Type=GetIntFromAPref(ActionNumber, "Type", TempString);
	if(Action->Type==HKACTIONTYPE_SetStatus)
	{
		Action->Info.State=GetIntFromAPref(ActionNumber, "State", TempString);
		Action->Info.MessageState=GetIntFromAPref(ActionNumber, "MessageState", TempString);
		Action->Info.Message=g_strdup(GetStringFromAPref(ActionNumber, "Message", TempString));
	}
	else //HKACTIONTYPE_Other
		Action->Info.State=GetIntFromAPref(ActionNumber, "Action", TempString);

	g_string_free(TempString, TRUE);
	return Action;
}

//Returns the default settings, and the number of them into NumSettings
const HotKeyAction* GetDefaultSettings(int *NumSettings)
{
	int Num=*NumSettings=sizeof(DefaultSettings)/sizeof(HotKeyAction);

	//If the keys have not been remapped to OS keycodes yet, do so
	static int RemapKeys=0;
	if(!RemapKeys)
	{
		for(int i=0;i<Num;i++)
		{
			char KeyName[2]={DefaultSettings[i].Key.Key, 0}; //Turn into a Key Name string
			GdkKeymapKey* KeyCodes;
			gint NumKeyCodes;
			gdk_keymap_get_entries_for_keyval(NULL, gdk_keyval_from_name(KeyName), &KeyCodes, &NumKeyCodes);
			if(NumKeyCodes) //If KeyCodes are returned, take the first one given. This should probably check for group:0, level:0, but meh.
				DefaultSettings[i].Key.Key=KeyCodes[0].keycode;
			g_free(KeyCodes);
		}
		RemapKeys=1;
	}

	return DefaultSettings;
}

//Internal functions: Get/Set/Delete action related preferences
static const char* FormatAPrefName(int ActionNum, const char* ActionMember, GString *TempString) //Formats the preference name
{
	g_string_printf(TempString, "Action%i_%s", ActionNum, ActionMember);
	return TempString->str;
}
static int         GetIntFromAPref   (int AC,const char* AM,GString *TS)               {return PrefsGetInt   (FormatAPrefName(AC,AM,TS));   } //Retreive an integer
static const char* GetStringFromAPref(int AC,const char* AM,GString *TS)               {return PrefsGetString(FormatAPrefName(AC,AM,TS));   } //Retreive a string
static void        SetAPrefFromInt   (int AC,const char* AM,GString *TS,int         NV){return PrefsSetInt   (FormatAPrefName(AC,AM,TS),NV);} //Save an integer
static void        SetAPrefFromString(int AC,const char* AM,GString *TS,const char* NV){return PrefsSetString(FormatAPrefName(AC,AM,TS),NV);} //Save a string
static void        RemoveAPref       (int AC,const char* AM,GString *TS)               {return PrefsRemove   (FormatAPrefName(AC,AM,TS));   } //Delete a preference
