//File Description: The configuration dialog
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h> //Modifier Key constants
#include <malloc.h>
#include "Actions.h"
#include "Config.h"
#include "Environment.h" //SetOtherOption

//Different State Names (Must be parallel to the arrays in Settings.h)
static const char* StatusStateNames          []={"KEEP CURRENT", "Available", "Away", "Do Not Disturb", "Invisible", "Offline"}; //Status States
static const char* SetStatusMessageStateNames[]={"Keep Current Message", "Static Message", "Last Non-Empty Message", "Prompt with Static Message", "Prompt with Last Non-Empty"}; //Set Status Message States
static const char* OtherHotKeyNames          []={"Focus Buddy List", "Hide Buddy List", "Set Buddy List as Bottom Window", "Toggle Taskbar Visibility"}; //Other Hot Keys
static const char* OtherOptionNames          []={"Buddy List Taskbar Icon Is Hidden On Startup"}; //Other Options

//About information
       const char About_ProgramName     []="Status Hot Keys";
       const char About_Version         []="1.5.1";
static const char About_Copyright       []="Jeffrey Riaboy (Hyrulian Productions)";
       const char About_ShortDescription[]="Global shortcut keys to set status and manipulate the buddy window visibility";
       const char About_LongDescription []="Dynamically configure any global hot key to: Set availability status (message can be chosen many ways including via a popup); Move pidgin Buddy List to top/bottom of windows; Hide the Buddy List window or taskbar window";
static const char About_License         []="Dakusan License v2.0 (See LICENSE.txt [including Addendum])";
       const char About_Author          []="Jeffrey Riaboy (http://www.castledragmire.com)";
       const char About_Website         []="http://www.castledragmire.com/Projects/Status_Hot_Keys";

//The GUI Widgets (Only the ones that need to be called on after creation)
static GtkWidget *ActionListScroller, *ActionList, *ShortcutKeyBox, *LocalOnlyCheck, *StatusStateBox, *StatusMessageStateBox, *StatusMessage, *OtherHotKeysBox;

//Functions
static void AddItemToActionList(HotKeyAction *TheAction); //Add an action to the display list
static void ModifyActionList(GtkWidget *Widget, gpointer DeleteOrEdit); //User clicked delete/edit button
static gboolean GetHotKey(GtkWidget *Widget, GdkEventKey *Event, gpointer Data); //Get the user pressed key from the ShortcutKeyBox
static gboolean GetHotKey_Release(GtkWidget *Widget, GdkEventKey *Event, gpointer Data); //Make sure the user selected a HotKey from the ShortcutKeyBox
static void AddAction(GtkWidget *Widget, gpointer Data); //Add an action
inline GtkWidget* InitWidget(GtkWidget *Widget, gint Width, gint Height, GtkWidget *Parent, gboolean Expand, gboolean Fill); //Initialize a widget by setting its size and parent, and then returning it. This is just a passthrough to gtk_widget_set_size_request & gtk_box_pack_start
static GtkWidget* LayoutConfigDialog(void); //Create the configuration dialog
static void StatusMessageStateBoxChanged(GtkWidget *Widget, gpointer Data); //When the status message state changes...
static void LocalOnlyChanged(GtkWidget *Widget, gpointer Data); //When the "Local Only" checkbox is pressed
static void OtherOptionChecked(GtkWidget *Widget, gpointer Data); //When an "Other Options" checkbox is pressed
static void ShowAbout(GtkWidget *Widget, gpointer Data); //Show the about dialog

//Other stuff
static const GdkColor StatusMessageNotUsed={0, 0xAAAA, 0xAAAA, 0xAAAA}; //Gray
static HotKey CurrentKey={0,0}; //The current key in the ShortcutKeyBox

//Loads the configuration dialog
GtkWidget* LoadConfigDialog(void)
{
	GtkWidget *Frame=LayoutConfigDialog(); //Create the configuration dialog

	//Copy the Actions to the list box
	for(GList *TraverseActions=Actions;TraverseActions;TraverseActions=g_list_next(TraverseActions))
		AddItemToActionList((HotKeyAction*)TraverseActions->data);

	gtk_widget_show_all(Frame); //Show the actions

	return Frame;
}

//Return a string representation of a HotKey
GString* FormatKeyName(HotKey Key)
{
	//Determine the string for the key's modifiers
	GString *KeyString=g_string_new(NULL);
	const char* KeyModifiers[]={"Ctrl+", "Alt+", "Shift+", "Win+"};
	for(int i=0;i<4;i++)
		if(Key.Modifiers&(1<<i))
			KeyString=g_string_append(KeyString, KeyModifiers[i]);

	//Convert keycode to a keyval
	GdkKeymapKey ConvertMap={Key.Key, 0, 0};
	guint KeyVal=gdk_keymap_lookup_key(NULL, &ConvertMap);

	//Convert key name; first by checking against a list of names (These are names I wanted to modify because I didn't like GTK's returns)
	typedef struct _KeyName
	{
		unsigned short Code; //This should be a gunit, but all values I've used are known as <=2 bytes
		const char* Name;
	} KeyName;
	const KeyName KeyNames[]={{0x0,""},{0xFF1B,"Esc"},{0xFF63,"Ins"},{0xFFFF,"Del"},{0xFF55,"PageUp"},{0xFF56,"PageDown"},{0x60,"`"},{0x2D,"-"},{0x3D,"="},{0xFFAF,"Num/"},{0xFFAA,"Num*"},{0xFFAD,"Num-"},{0xFFAB,"Num+"},{0x5B,"["},{0x5D,"]"},{0x5C,"\\"},{0x3B,";"},{0x27,"'"},{0x2C,","},{0x2E,"."},{0x2F,"/"}};
	const char NumKeyNames=sizeof(KeyNames)/sizeof(KeyName);
	char* MyKeyName=NULL;
	for(int i=0;i<NumKeyNames;i++)
		if(KeyNames[i].Code==KeyVal)
		{
			MyKeyName=(char*)KeyNames[i].Name;
			break;
		}
	//If no non-list names are found, check against lists
	char Buffer[8];
	if(KeyVal>=0xFFB0 && KeyVal<=0xFFB0+9) //NUMPAD0-9
	{
		g_strlcpy(MyKeyName=Buffer, "NUMPAD0", 8);
		Buffer[6]=KeyVal-0xFFB0+'0';
	}

	return g_string_append(KeyString, MyKeyName!=NULL ? MyKeyName : gdk_keyval_name(KeyVal)); //If name is not specified locally, use gdk to evaluate
}

//Retrieves the information text of an action
GString* GetActionText(HotKeyAction *Action)
{
	GString *ActionText=g_string_new(NULL);

	if(Action->Type==HKACTIONTYPE_Other) //Other Action
			g_string_printf(ActionText, "Other Action: %s", OtherHotKeyNames[Action->Info.State]);
	else //Set Status Action
	{
		g_string_printf(ActionText, "%s : %s", StatusStateNames[Action->Info.State], SetStatusMessageStateNames[Action->Info.MessageState]);
		if(Action->Info.MessageState==SSMS_STATIC || Action->Info.MessageState==SSMS_POPUP)
			g_string_append_printf(ActionText, " : %s", Action->Info.Message);
	}
	return ActionText;
}

//Add an action to the display list
static void AddItemToActionList(HotKeyAction *Action)
{
	//Get the information strings
	GString *TheKey=FormatKeyName(Action->Key);
	GString *ActionText=GetActionText(Action);

	//Create the display to show the hotkey and its action
	GtkWidget *hbox=gtk_hbox_new(FALSE, 0);
	gtk_misc_set_alignment(GTK_MISC(InitWidget(gtk_label_new(TheKey->str), 125, -1, hbox, FALSE, FALSE)), 0, 0);
	gtk_misc_set_alignment(GTK_MISC(InitWidget(gtk_label_new(ActionText->str), -1, -1, hbox, TRUE, TRUE)), 0, 0);

	//Create the list item and add it to the list
	GtkWidget *TheNewItem=gtk_list_item_new();
	gtk_container_add(GTK_CONTAINER(TheNewItem), hbox);
	gtk_list_append_items(GTK_LIST(ActionList), g_list_append(NULL, TheNewItem)); //Add the item to the listbox
	gtk_widget_show_all(TheNewItem);

	//Free strings
	g_string_free(TheKey, TRUE);
	g_string_free(ActionText, TRUE);
}

//User clicked delete/edit button
static void ModifyActionList(GtkWidget *Widget, gpointer DeleteOrEdit)
{
	//Get the selected item, exiting if nothing is selected
	GList *TheSelection=g_list_first(((GtkList*)ActionList)->selection); //Get the first item of the selection list
	if(!TheSelection) //If nothing is selected, exit
		return;
	gint SelectedItem=gtk_list_child_position(GTK_LIST(ActionList), (GtkWidget*)TheSelection->data); //Selected item index
	GList *TempList=g_list_append(NULL, TheSelection->data);
	gtk_list_remove_items(GTK_LIST(ActionList), TempList); //Delete the selected item out of the listbox
	g_list_free(TempList);

	//If the action is to "edit", move the action's data into the fields
	HotKeyAction *Action=(HotKeyAction*)g_list_nth_data(Actions, SelectedItem);
	if(DeleteOrEdit==0) //0==edit
	{
		GString *KeyName=FormatKeyName(CurrentKey=Action->Key); //Also update the currently set key
		gtk_entry_set_text(GTK_ENTRY(ShortcutKeyBox), KeyName->str);
		g_string_free(KeyName, TRUE);

		//gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(LocalOnlyCheck), Action->LocalOnly ? TRUE : FALSE); //*Not currently supported

		if(Action->Type==HKACTIONTYPE_Other) //Other Action
			gtk_combo_box_set_active(GTK_COMBO_BOX(OtherHotKeysBox), Action->Info.State);
		else //Set Status Action
		{
			gtk_combo_box_set_active(GTK_COMBO_BOX(StatusStateBox), Action->Info.State);
			gtk_combo_box_set_active(GTK_COMBO_BOX(StatusMessageStateBox), Action->Info.MessageState);
			if(Action->Info.MessageState==SSMS_STATIC || Action->Info.MessageState==SSMS_POPUP)
				gtk_entry_set_text(GTK_ENTRY(StatusMessage), Action->Info.Message);
		}
	}

	//Delete the action from the Actions list
	DeleteAction(SelectedItem, TRUE);
}

//Get the user pressed key from the ShortcutKeyBox
static gboolean GetHotKey(GtkWidget *Widget, GdkEventKey *Event, gpointer Data) //Data==0 for a press, and 1 for a release (called from GetHotKey_Release)
{
	//Determine the pressed modifier keys
	const guint GDKControlMasks[]={GDK_CONTROL_MASK, GDK_MOD1_MASK, GDK_SHIFT_MASK, GDK_MOD4_MASK};
	const guint ControlMasks[]={MODK_CONTROL, MODK_ALT, MODK_SHIFT, MODK_WIN};
	const guint ControlKeyValues[]={GDK_Control_L, GDK_Control_R, GDK_Alt_L, GDK_Alt_R, GDK_Shift_L, GDK_Shift_R, GDK_Super_L, GDK_Super_R}; //Each control key has 2 possible hardware values
	const guint NumControlKeys=sizeof(GDKControlMasks)/sizeof(guint);
	guint ControlKeys=0, NewKey=Event->hardware_keycode, KeyVal=Event->keyval; //Store locally what has been pressed

	for(int i=0;i<NumControlKeys;i++) //Try each state and hardware code against the predefined known values to determine what control keys currently are being pressed
		if(KeyVal==ControlKeyValues[i*2] || KeyVal==ControlKeyValues[i*2+1]) //If control key has just been pressed or released, set the key to 0, and set as a modifier if pressed
		{
			if(Data==0) //Only set the key if coming from a key press
				ControlKeys|=ControlMasks[i];
			KeyVal=NewKey=0;
		}
		else if(Event->state & GDKControlMasks[i]) //If control key is down, add it to the modifier list
			ControlKeys|=ControlMasks[i];

	//Store the newly pressed key
	CurrentKey.Modifiers=ControlKeys;
	CurrentKey.Key=NewKey;

	//Set the textbox to the current name
	GString *KeyName=g_string_prepend(FormatKeyName(CurrentKey), " "); //Prepand a " " so cursor does not interfere //*Would be better if cursor could be removed?
	gtk_entry_set_text(GTK_ENTRY(Widget), KeyName->str);
	g_string_free(KeyName, TRUE);

	return FALSE;
}

//Make sure the user selected a HotKey from the ShortcutKeyBox
static gboolean GetHotKey_Release(GtkWidget *Widget, GdkEventKey *Event, gpointer Data)
{
	if(CurrentKey.Key!=0) //Nothing to do if a key has been previously pressed
		return FALSE;

	//Determine the current state of the hot keys
	GetHotKey(Widget, Event, (gpointer)1);

	if(CurrentKey.Key==0 && CurrentKey.Modifiers==0) //If nothing is being pressed, set text back to default state
		gtk_entry_set_text(GTK_ENTRY(ShortcutKeyBox), "Shortcut Key");

	return FALSE;
}

//Add an action
static void AddAction(GtkWidget *Widget, gpointer Data) //Data==0 for Set Status, 1 for Other
{
	//Make sure state is valid
	if(CurrentKey.Key==0)
	{
		ShowDialogBox("Cannot add action", "Invalid Shortcut Key");
		return;
	}

	//Create the action
	HotKeyAction *Action=(HotKeyAction*)malloc(sizeof(HotKeyAction));
	Action->Key=CurrentKey;
	Action->LocalOnly=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(LocalOnlyCheck));
	Action->Type=(Data==0 ? HKACTIONTYPE_SetStatus : HKACTIONTYPE_Other);
	if(Data==0) //Process status message state
	{
		Action->Info.State=gtk_combo_box_get_active(GTK_COMBO_BOX(StatusStateBox));
		int MessageState=Action->Info.MessageState=gtk_combo_box_get_active(GTK_COMBO_BOX(StatusMessageStateBox));
		Action->Info.Message=g_strdup(MessageState==SSMS_STATIC || MessageState==SSMS_POPUP ? gtk_entry_get_text(GTK_ENTRY(StatusMessage)) : "");
	}
	else //Process other state
		Action->Info.State=gtk_combo_box_get_active(GTK_COMBO_BOX(OtherHotKeysBox));

	//Add it to the proper lists
	AddItemToActionList(Action);
	SaveAction(Action, TRUE);

	//Scroll to the end of the list
	//gtk_list_scroll_vertical(GTK_LIST(ActionList), GTK_SCROLL_END, 0); //I don't know why this doesn't work but it would remove a bit of code and some extra variables if it did... :-(
	GtkAdjustment *VAdj=gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(ActionListScroller));
	VAdj->value=VAdj->upper;
	gtk_scrolled_window_set_vadjustment(GTK_SCROLLED_WINDOW(ActionListScroller), VAdj);
}

//Initialize a widget by setting its size and parent, and then returning it
inline GtkWidget* InitWidget(GtkWidget *Widget, gint Width, gint Height, GtkWidget *Parent, gboolean Expand, gboolean Fill)
{
	gtk_widget_set_size_request(Widget, Width, Height);
	gtk_box_pack_start(GTK_BOX(Parent), Widget, Expand, Fill, 0);
	return Widget;
}

//Create the configuration dialog
static GtkWidget* LayoutConfigDialog(void)
{
	GtkWidget *vbox=gtk_vbox_new(FALSE, 3); //Create the main vbox

	//Bold
	PangoAttribute *Bold=pango_attr_weight_new(PANGO_WEIGHT_BOLD);
	PangoAttrList *AttrList=pango_attr_list_new();
	pango_attr_list_insert(AttrList, Bold);

	//Create header labels
	const char* HeaderLabelTexts[3]={"Currently Bound Hot Keys", "Add Hot Key", "Other Options"};
	GtkWidget *HeaderLabels[3];
	for(int i=0;i<3;i++)
	{
		//Move down 10 pixels (except on the first label) and center
		HeaderLabels[i]=gtk_alignment_new(.5,0,0,0);
		gtk_alignment_set_padding(GTK_ALIGNMENT(HeaderLabels[i]), i==0 ? 0 : 10, 0, 0, 0);

		//Create the label
		GtkWidget *NewItem=gtk_label_new(HeaderLabelTexts[i]);
		gtk_container_add(GTK_CONTAINER(HeaderLabels[i]), NewItem);
		gtk_label_set_attributes(GTK_LABEL(NewItem), AttrList);
	}

	//Create the "Currently Bound Hot Keys" section
		gtk_box_pack_start(GTK_BOX(vbox), HeaderLabels[0], FALSE, FALSE, 0); //Label
		//Action list with scrollbar
			ActionListScroller=InitWidget(gtk_scrolled_window_new(NULL, NULL), -1, 65, vbox, TRUE, TRUE);
			gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(ActionListScroller), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
			gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(ActionListScroller), ActionList=gtk_list_new());
		//Buttons section
			GtkWidget *hbox=InitWidget(gtk_hbox_new(FALSE, 0), -1, -1, vbox, FALSE, FALSE);
			//Buttons
			const char* HotKeyButtons[]={"Edit", "Delete"};
			for(int i=0;i<2;i++)
				gtk_signal_connect(GTK_OBJECT(InitWidget(gtk_button_new_with_label(HotKeyButtons[i]), 105, -1, hbox, TRUE, FALSE)), "pressed", GTK_SIGNAL_FUNC(ModifyActionList), (gpointer)i);

	//Create the "Add Hot Key" section
		gtk_box_pack_start(GTK_BOX(vbox), HeaderLabels[1], FALSE, FALSE, 0); //Label
		GtkWidget *table=InitWidget(gtk_table_new(2, 4, FALSE), -1, -1, vbox, FALSE, FALSE);
		gtk_table_set_col_spacing(GTK_TABLE(table), 0, 5);
		//Shortcut Key Section
			gtk_table_attach(GTK_TABLE(table), gtk_label_new("Shortcut Key:"), 0, 1, 0, 1, (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0); //Label
			//Shortcut key box
			hbox=gtk_hbox_new(FALSE, 3);
			gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 0, 1, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)0, 0, 0);
			ShortcutKeyBox=InitWidget(gtk_entry_new(), -1, -1, hbox, TRUE, TRUE);
			gtk_entry_set_text(GTK_ENTRY(ShortcutKeyBox), "Shortcut Key");
			gtk_entry_set_editable(GTK_ENTRY(ShortcutKeyBox), FALSE);
			gtk_entry_set_alignment(GTK_ENTRY(ShortcutKeyBox), 0.5);
			g_signal_connect(GTK_OBJECT(ShortcutKeyBox), "key_press_event", G_CALLBACK(GetHotKey), NULL);
			g_signal_connect(GTK_OBJECT(ShortcutKeyBox), "key_release_event", G_CALLBACK(GetHotKey_Release), NULL);
			//Local Only
			LocalOnlyCheck=InitWidget(gtk_check_button_new_with_label("Local Only"), -1, -1, hbox, FALSE, FALSE);
			g_signal_connect(GTK_OBJECT(LocalOnlyCheck), "toggled", G_CALLBACK(LocalOnlyChanged), 0);
		//Set Status Section
			gtk_table_attach(GTK_TABLE(table), gtk_label_new("1) Set Status:"), 0, 1, 1, 3, (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0); //Label
			//Row 1
			hbox=gtk_hbox_new(FALSE, 3);
			gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 1, 2, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)0, 0, 0);
			//Statuses
			StatusStateBox=InitWidget(gtk_combo_box_new_text(), -1, -1, hbox, FALSE, FALSE);
			for(int i=0;i<STATUS_LENGTH;i++)
				gtk_combo_box_append_text(GTK_COMBO_BOX(StatusStateBox), StatusStateNames[i]);
			gtk_combo_box_set_active(GTK_COMBO_BOX(StatusStateBox), 0);
			//Set Status Message States
			StatusMessageStateBox=InitWidget(gtk_combo_box_new_text(), -1, -1, hbox, TRUE, TRUE);
			for(int i=0;i<SMSS_LENGTH;i++)
				gtk_combo_box_append_text(GTK_COMBO_BOX(StatusMessageStateBox), SetStatusMessageStateNames[i]);
			gtk_combo_box_set_active(GTK_COMBO_BOX(StatusMessageStateBox), 1);
			g_signal_connect(GTK_OBJECT(StatusMessageStateBox), "changed", G_CALLBACK(StatusMessageStateBoxChanged), NULL);
			//Row 2
			hbox=gtk_hbox_new(FALSE, 3);
			gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 2, 3, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)0, 0, 0);
			//Status Message
			StatusMessage=InitWidget(gtk_entry_new(), 1, -1, hbox, TRUE, TRUE);
			gtk_entry_set_text(GTK_ENTRY(StatusMessage), "Status Message Goes here");
			//Add Button
			g_signal_connect(GTK_OBJECT(InitWidget(gtk_button_new_with_label("Add"), -1, -1, hbox, FALSE, FALSE)), "pressed", GTK_SIGNAL_FUNC(AddAction), 0);
		//Set Other Section
			gtk_table_attach(GTK_TABLE(table), gtk_label_new("2) Set Other:"), 0, 1, 3, 4, (GtkAttachOptions)0, (GtkAttachOptions)0, 0, 0); //Label
			hbox=gtk_hbox_new(FALSE, 3);
			gtk_table_attach(GTK_TABLE(table), hbox, 1, 2, 3, 4, (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), (GtkAttachOptions)0, 0, 0);
			//"Add Other Hot Keys" List Box
			OtherHotKeysBox=InitWidget(gtk_combo_box_new_text(), -1, -1, hbox, TRUE, TRUE);
			for(int i=0;i<OHK_LENGTH;i++)
				gtk_combo_box_append_text(GTK_COMBO_BOX(OtherHotKeysBox), OtherHotKeyNames[i]);
			gtk_combo_box_set_active(GTK_COMBO_BOX(OtherHotKeysBox), 0);
			//Add button
			g_signal_connect(GTK_OBJECT(InitWidget(gtk_button_new_with_label("Add"), -1, -1, hbox, FALSE, FALSE)), "pressed", GTK_SIGNAL_FUNC(AddAction), (gpointer)1);
	//Create the "Other Options" section
		gtk_box_pack_start(GTK_BOX(vbox), HeaderLabels[2], FALSE, FALSE, 0); //Label
		//Other Options
			for(int i=0;i<OO_LENGTH;i++)
			{
				const char* OtherOptionPrefNames[]={"Taskbar_At_Startup"}; //List of other options' preferences
				GtkWidget* NewCheckButton=InitWidget(gtk_check_button_new_with_label(OtherOptionNames[i]), -1, -1, vbox, FALSE, FALSE); //Create the button
				gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(NewCheckButton), PrefsGetBool(OtherOptionPrefNames[i])); //Set its toggle state
				g_signal_connect(GTK_OBJECT(NewCheckButton), "toggled", GTK_SIGNAL_FUNC(OtherOptionChecked), (gpointer)i); //When used, update the preference
			}
		//About button
		gtk_signal_connect(GTK_OBJECT(InitWidget(gtk_button_new_with_label("About"), 100, -1, InitWidget(gtk_hbox_new(FALSE,0), -1, -1, vbox, FALSE, FALSE), TRUE, FALSE)), "pressed", GTK_SIGNAL_FUNC(ShowAbout), 0);

	//Free memory
	pango_attr_list_unref(AttrList);

	return vbox;
}

//When the status message state changes...
static void StatusMessageStateBoxChanged(GtkWidget *Widget, gpointer Data)
{
	//Set the "Message" box to enabled depending on the state
	gint CurrentState=gtk_combo_box_get_active(GTK_COMBO_BOX(Widget));
	gboolean UsesMessage=(CurrentState==SSMS_STATIC || CurrentState==SSMS_POPUP);
	gtk_entry_set_editable(GTK_ENTRY(StatusMessage), UsesMessage);
	gtk_widget_modify_base(StatusMessage, GTK_STATE_NORMAL, UsesMessage ? NULL : &StatusMessageNotUsed);
}

//When the "Local Only" checkbox is pressed
static void LocalOnlyChanged(GtkWidget *Widget, gpointer Data)
{
	GTK_TOGGLE_BUTTON(Widget)->active=FALSE;
	ShowDialogBox("Unsupported", "\"Local only\" is currently not supported and is ignored");
}

//When an "Other Options" checkbox is pressed
static void OtherOptionChecked(GtkWidget *Widget, gpointer Data)
{
	const char* OtherOptionPrefNames[]={"Taskbar_At_Startup"}; //List of other options' preferences
	PrefsSetBool(OtherOptionPrefNames[(int)Data], GTK_TOGGLE_BUTTON(Widget)->active); //Update the preference
}

//Show the about dialog
static void ShowAbout(GtkWidget *Widget, gpointer data)
{
	gtk_show_about_dialog(NULL, "name", "About Status Hot Keys", "program-name", About_ProgramName, "version", About_Version, "copyright", About_Copyright, "comments", About_LongDescription, "license", About_License, "website", About_Website, NULL);
	//"authors", About_Author :: NOT INCLUDED because it is crashing GTK O.o
}

//Shows a message box
void ShowDialogBox(const char* Title, const char* Text)
{
	GtkWidget *Dialog=gtk_message_dialog_new(NULL, (GtkDialogFlags)0, GTK_MESSAGE_WARNING, GTK_BUTTONS_OK, "%s", Title);
	gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(Dialog), "%s", Text);
	gtk_dialog_run(GTK_DIALOG(Dialog));
	gtk_widget_destroy(Dialog);
}
