//File Description: Interface with Linux (X11)

#include <X11/Xlib.h>
#include <gdk/gdkx.h>
#include <gtk/gtkwindow.h>
#include <gdk/gdkwindow.h>
#include "OSInterface.h"
#include "Environment.h"
#include "Actions.h"

//A blank window used for registering hot keys
GtkWidget *DummyWindow;

//Internal functions for processing a global hotkey
static GdkFilterReturn gdk_filter(GdkXEvent *xEvent, GdkEvent *Event, gpointer Data);

//Called on plugin load to prepare the OS Interface
void InitOSInterface(void)
{
	//Create a dummy window and position it off the screen
	DummyWindow=gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_window_move(GTK_WINDOW(DummyWindow), -100, -100);
	gtk_window_resize(GTK_WINDOW(DummyWindow), 1, 1);

	//Listen for events (for hot key filtering)
	gdk_window_add_filter(gtk_widget_get_root_window(DummyWindow), gdk_filter, 0);
}

//Called on plugin unload to close the OS Interface
void CloseOSInterface(void)
{
	//Destroy the dummy window
	gdk_window_remove_filter(gtk_widget_get_root_window(DummyWindow), gdk_filter, 0);
	gtk_widget_destroy(DummyWindow);
}

//Handle registering or unregistering a hot key. Returns success
static const guint XControlMasks[]={ControlMask, Mod1Mask, ShiftMask, Mod4Mask};
static const guint ControlMasks[]={MODK_CONTROL, MODK_ALT, MODK_SHIFT, MODK_WIN};
static gboolean HandleKeyRegistration(gboolean Registering, int KeyCode, int Modifiers)
{
	//Get X information for hooking
	Display *TheDisplay=(Display*)GDK_DISPLAY_XDISPLAY(gtk_widget_get_display(DummyWindow));
	Window Root=GDK_WINDOW_XID(gtk_widget_get_root_window(DummyWindow));

	//Determine the XModifier keys
	unsigned char XModifiers=0;
	for(int i=0;i<4;i++)
		if(Modifiers&ControlMasks[i])
			XModifiers|=XControlMasks[i];

	if(Registering) //Register the key
	{
		gdk_error_trap_push(); //We have to trap an x errors, because one is thrown if the key is already hooked
		XGrabKey(TheDisplay, KeyCode, XModifiers, Root, True, GrabModeAsync, GrabModeAsync); //Register the key
		gdk_flush(); //Prepare to finish trapping errors
		if(gdk_error_trap_pop()) //Stop trapping errors and see if one occured, and if it did, return unsuccessful
			return 0;
	}
	else //Unregister the key
		XUngrabKey(TheDisplay, KeyCode, XModifiers, Root);

	return 1; //Return success
}

//Register the hot key in the OS for a callback. Returns an ID for the hotkey, or 0 on failure
int RegisterHotKeyAction(int HotKeyNumber, int Key, int Modifiers)
{
	return (HandleKeyRegistration(TRUE, Key, Modifiers) ? HotKeyNumber+1 : 0);
}

//Unregister the hot key in the OS
void UnRegisterHotKeyAction(int HotKeyID)
{
	//Find the action
	GList *TraverseActions=Actions;
	for(int Counter=1;Counter<HotKeyID;TraverseActions=g_list_next(TraverseActions),Counter++);
	HotKeyAction *Action=(HotKeyAction*)TraverseActions->data;
	HandleKeyRegistration(FALSE, Action->Key.Key, Action->Key.Modifiers);
}

//Toggle the buddy list in the taskbar
void ToggleBuddyListInTaskbar(enum ToggleTaskbar_Type Type)
{
	GtkWindow *TheWindow=GTK_WINDOW(GetGTKBuddyListWindow());
	gtk_window_set_skip_taskbar_hint(TheWindow, (Type==TaskbarType_Swap ? !gtk_window_get_skip_taskbar_hint(TheWindow) : (Type==TaskbarType_Visible ? FALSE : TRUE)));
}

//Moves the buddy list to above or below all other windows
void SetBuddyListZOrder(gboolean OnTop)
{
	GdkWindow *BList=GetGDKBuddyListWindow();
	gtk_widget_show(DummyWindow); //The only way I could get this to work in Linux was by showing the Dummy window first >.<
	gdk_window_raise(BList); //Raise the parent no matter what, so it can lower itself later if need be
	gdk_window_focus(BList, 0);
	if(!OnTop)
		gdk_window_lower(BList);
	gtk_widget_hide(DummyWindow);
}

//Internal function for processing a global hotkey
static GdkFilterReturn gdk_filter(GdkXEvent *EventData, GdkEvent *Event, gpointer Data)
{
	//Make sure this is a key event
	if(((XEvent*)EventData)->type!=KeyPress)
		return GDK_FILTER_CONTINUE;

	//Extract the key modifiers
	XKeyEvent *KeyEvent=(XKeyEvent*)EventData;
	unsigned char Modifiers=0;
	for(int i=0;i<4;i++)
		if(KeyEvent->state&XControlMasks[i])
			Modifiers|=ControlMasks[i];

	//Find the Action from the pressed key
	int Counter=0;
	for(GList *TraverseActions=Actions;TraverseActions;TraverseActions=g_list_next(TraverseActions),Counter++)
	{
		HotKeyAction *Action=(HotKeyAction*)TraverseActions->data;
		if(Action->Key.Key==KeyEvent->keycode && Action->Key.Modifiers==Modifiers)
		{
			ProcessHotKey(Counter+1);
			return GDK_FILTER_REMOVE;
		}
	}

	//If action is not found, continue as normal
	return GDK_FILTER_CONTINUE;
}