//File Description: Interface with Windows

typedef void GtkWidget;

#include <windows.h>
#include <glib/gstring.h>
#include <gdk/win32/gdkwin32.h> //Get the window for manipulation (gdk_win32_drawable_get_handle)
#include "OSInterface.h"
#include "Environment.h"
#include "Actions.h"

//A blank window used for multiple internal operations
HWND DummyWindow=NULL;

//Internal Functions
static HWND GetBuddyListWindow(void); //Get the buddy list window's handle
static LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam); //Process the dummy window's messages to find global hotkeys

//Called on plugin load to prepare the OS Interface
void InitOSInterface(void)
{
	//Create a dummy window to receive the hot keys
	WNDCLASS wc={0, WndProc, 0L, 0L, GetModuleHandle(NULL), NULL, NULL, NULL, NULL, "PidginHotKeyDummy"};
	RegisterClass(&wc);
	DummyWindow=CreateWindow("PidginHotKeyDummy", "", 0, 0, 0, 0, 0, NULL, NULL, GetModuleHandle(NULL), 0);
}

//Called on plugin unload to close the OS Interface
void CloseOSInterface(void)
{
	ToggleBuddyListInTaskbar(TaskbarType_Visible); //Make sure the buddy list isn't a child of the dummy window

	//Destroy the dummy window
	DestroyWindow(DummyWindow);
	DummyWindow=NULL;
}

//Register the hot key in the OS for a callback. Returns an ID for the hotkey, or 0 on failure
int RegisterHotKeyAction(int HotKeyNumber, int KeyCode, int Modifiers)
{
	//Prepare the name for the Atom for the hotkey
	GString *HotKeyName=g_string_new(NULL);
	g_string_printf(HotKeyName, "StatusHotKeys%i", HotKeyNumber);

	//Register the hot key
	int ReturnIdentifier;
	int Success=RegisterHotKey(DummyWindow, ReturnIdentifier=GlobalAddAtom(HotKeyName->str),
			(Modifiers&MODK_CONTROL ? MOD_CONTROL : 0)|
			(Modifiers&MODK_ALT     ? MOD_ALT     : 0)|
			(Modifiers&MODK_SHIFT   ? MOD_SHIFT   : 0)|
			(Modifiers&MODK_WIN     ? MOD_WIN     : 0),
		KeyCode);
	g_string_free(HotKeyName, TRUE);

	if(Success) //If successful, return the identifier
		return ReturnIdentifier;

	//Otherwise, delete the adam and return unsuccessful
	GlobalDeleteAtom(ReturnIdentifier);
	return 0;
}

//Unregister the hot key in the OS
void UnRegisterHotKeyAction(int HotKeyID)
{
	UnregisterHotKey(DummyWindow, HotKeyID); //Remove the hotkey
	GlobalDeleteAtom(HotKeyID); //Delete the atom
}

//Toggle the buddy list in the taskbar
void ToggleBuddyListInTaskbar(enum ToggleTaskbar_Type Type)
{
	//Get the window, and exit if it has not been created
	HWND BuddyListHwnd=GetBuddyListWindow();
	if(!BuddyListHwnd)
		return;

	//Toggle displaying the buddy list in the taskbar
	BOOL IsCurrentlyHidden=((HWND)GetWindowLong(BuddyListHwnd, GWL_HWNDPARENT)==DummyWindow); //Determine current taskbar status
	BOOL IsHiding=(Type==TaskbarType_Swap ? !IsCurrentlyHidden : (Type==TaskbarType_Visible ? FALSE : TRUE));
	if(IsHiding) //If about to hide...
		SetClassLong(DummyWindow, GCL_HICON, GetClassLong(BuddyListHwnd, GCL_HICON)); //Copy the buddy list icon to our dummy window so it shows when alt tabbing

	SetWindowLong(BuddyListHwnd, GWL_HWNDPARENT, IsHiding ? (LONG)DummyWindow : 0); //Using GWL_HWNDPARENT is technically not safe in SetWindowLong, but it is the only way to do it with proper results right now

	//Force window reactiviation to toggle the taskbar
	SetForegroundWindow(DummyWindow);
	SetForegroundWindow(BuddyListHwnd);
}

//Moves the buddy list to above or below all other windows
void SetBuddyListZOrder(gboolean OnTop)
{
	HWND BuddyListHwnd=GetBuddyListWindow();
	BOOL IsCurrentlyHidden=((HWND)GetWindowLong(BuddyListHwnd, GWL_HWNDPARENT)==DummyWindow); //Determine current taskbar status
	HWND HwndToUse=(IsCurrentlyHidden ? DummyWindow : BuddyListHwnd); //If hidden, we need to use the dummy window, otherwise, the buddy list window itself
	SetWindowPos(HwndToUse, OnTop ? HWND_TOP : HWND_BOTTOM, 0, 0, 0, 0, SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOSIZE);
}

//Process the dummy window's messages to find global hotkeys
static LRESULT CALLBACK WndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	if(uMsg==WM_HOTKEY)
		ProcessHotKey((int)wParam);
	return DefWindowProc(hwnd, uMsg, wParam, lParam);
}

//Get the buddy list window's handle
static HWND GetBuddyListWindow(void)
{
	GdkWindow *TheWindow=GetGDKBuddyListWindow();
	return (!TheWindow ? NULL : (HWND)gdk_win32_drawable_get_handle(TheWindow)); //If window does not yet exist, return NULL, otherwise, Get the buddy list's HWND
}
